import { gql } from "apollo-boost";

export const USER_POLICY_EXISTS = gql`
query UserPolicyExists($emailId: String) {
    policy(where: {subject: {user: {email_id: {_eq: $emailId}}}}) {
        id
    }
  }
`;

export const POLICY_APP_RESOURCE_EXISTS = gql`
query PolicyAppResourceExists($policyId: uuid!, $appName: String, $productName: String) {
  policy_resource(where:{
    _and: [
    {policy_id:{_eq: $policyId}},
    {resource: {app: {name: {_eq: $appName}}}},
    {resource: {product: {name: {_eq: $productName}}}}
  ]
  }) {
       id,
       app_product_id
  }
}
`;

export const GET_USER_POLICY_TAXONOMY = gql`
query getUserPolicyTaxonomy($emailId: String, $appName: String, $productName: String, $taxonomyName: String) {
  policy_resource_taxon_allow(where: {
    _and: [
      {policyResource: {policy: {subject: {user: {email_id: {_eq: $emailId}}}}}},
      {policyResource: {resource: {app: {name: {_eq: $appName}}}}},
      {policyResource: {resource: {product: {name: {_eq: $productName}}}}},
      {taxonomy: {name: {_eq: $taxonomyName}}}
    ]
  }) {
    policy_resource_id,
    product_taxonomy_id,
    value
  }
}
`;

export const GET_USER_POLICY_ALL_TAXONOMY = gql`
query getUserPolicyTaxonomy($emailId: String, $appName: String, $productName: String, $taxonomyName: String) {
  policy_resource_taxon_allow(where: {
    _and: [
      {policyResource: {policy: {subject: {user: {email_id: {_eq: $emailId}}}}}},
      {policyResource: {resource: {app: {name: {_eq: $appName}}}}},
      {policyResource: {resource: {product: {name: {_eq: $productName}}}}},
    ]
  }) {
    policy_resource_id,
    product_taxonomy_id,
    taxonomy{
      name
    }
  }
}
`;

export const GET_PRODUCT_ALL_TAXONOMY = gql`
query PolicyProductTaxonomy($appProductId: uuid) {
  app_product(where: { id: { _eq: $appProductId } }) {
    product {
      taxonomies {
        id
        name
      }
    }
  }
}
`;

export const GET_USER_IDS = gql`
query GetUserId($emailIds: [String!]){
  user(where: {email_id: {_in: $emailIds}}) {
    email_id,
    id
  }
}
`;

export const GET_APP_PRODUCT_ID = gql`
query GetAppProductId($appName: String, $productName: String) {
  app_product(where:{
    _and: [
    	{app: {name:{_eq: $appName}}},
      {product: {name: {_eq: $productName}}}
    ]
  }) { 
       id
  }
}
`;

export const GET_PRODUCT_TAXONOMY_ID = gql`
query GetProductTaxonomyId($productName: String!, $productTaxonomyNames: [String!]) {
  product_taxonomy(where:{
    _and:[
      {product:{name:{_eq: $productName}}}
      {name:{_in:$productTaxonomyNames}}
    ]
  }) {
       id,
       name
  }
}
`;

export const CREATE_USER_POLICY = gql`
mutation PolicyNew($policyInsertInput: [policy_insert_input!]!) {
  insert_policy(objects: $policyInsertInput) {
               affected_rows
  }
}
`;

export const ADD_POLICY_RESOURCE = gql`
mutation AddPolicyResource($policyResourceInsertInput: [policy_resource_insert_input!]!) {
  insert_policy_resource(
      objects: $policyResourceInsertInput
      on_conflict: { constraint: policy_resource_id_key, update_columns: app_product_id }
  ) {
      affected_rows
  }
}
`;

export const ADD_POLICY_TAXONOMY = gql`
mutation TaxonAllowInsert($policyResourceTaxonAllowInsertInput: [policy_resource_taxon_allow_insert_input!]!) {
  insert_policy_resource_taxon_allow(
  objects: $policyResourceTaxonAllowInsertInput
  on_conflict: { constraint: policy_resource_taxon_allow_pkey, update_columns: [value] }
  ) {
  affected_rows
  }
}
`;

export const UPDATE_POLICY_TAXONOMY = gql`
mutation updateTaxonAllow($policyResourceId: uuid!, $productTaxonomyId:uuid!, $value: jsonb) {
  update_policy_resource_taxon_allow(where:{_and:[{policy_resource_id:{_eq:$policyResourceId}}, {product_taxonomy_id:{_eq: $productTaxonomyId}}]}, _set:{value: $value}) {
    affected_rows
  }
}
`;

export const GET_APP_TAXON_USERS = gql`
query GetAppTaxonUsers($appName: String, $productName: String, $productTaxonomyNames: [String!]) {
  policy_resource_taxon_allow(where: {
    _and: [
      {policyResource: {resource: {app: {name: {_eq: $appName}}}}},
      {policyResource: {resource: {product: {name: {_eq: $productName}}}}},
      {taxonomy: {name: {_in: $productTaxonomyNames}}}
    ]
  }) {
    policyResource{
      policy {
       subject {
         user {
          display_name, 
          email_id
         }
       }
     }
    }
    value
  }
}
`;

export const GET_APP_ALL_USERS = gql`
query GetAppAllUsers($appName: String, $productName: String) {
  policy_resource(where: {
    _and:[
      {resource: {app: {name: {_eq: $appName}}}},
      {resource: {product: {name: {_eq: $productName}}}},
      {policy: {kind: {_eq: "group"}}}
    ]
  }) {
        policy {
          subject {
            group {
              users {
                user {
                  email_id,
                  display_name
                }
              }
            }
          }
        }
    		taxon_allow {
          value,
          taxonomy {
            name
          }
        },
    	
      }
}
`;

export const GET_DC_APP_ALL_USERS = gql`
query GetAppAllUsers($appName: String, $productName: String) {
  policy_resource(
    where: {
      _and: [
        {
          _or: [
            { resource: { app: { name: { _eq: $appName } } } }
            { resource: { product: { name: { _eq: $productName } } } }
            { resource: { app: { name: { _in: ["fo-pies2","fo-portal"] } } } }
            { resource: { product: { name: { _in: ["fo-pie2-bff","fo-bff"] } } } }
          ]
        }
        { policy: { kind: { _eq: "group" } } }
      ]
    }
  ) {
    policy {
      subject {
        group {
          name
          users {
            user {
              email_id
              display_name
              policies {
                policy {
                  statements{
                    taxon_allow(where: {taxonomy: {name: {_eq: "manager-emailid"}}}){
                      value
                    	taxonomy{
                        name
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    taxon_allow {
      value
      taxonomy {
        name
      }
    }
  }
}
`;

export const GET_USER_GROUPS = gql`
query GetAppAllUsers($emailId: String, $appName: String, $productName: String) {
  policy(where: {
    _and:[
      {statements: {resource: {app: {name: {_eq: $appName}}}}},
      {statements: {resource: {product: {name: {_eq: $productName}}}}},
      {subject: {group : {users: {user: {email_id: {_eq: $emailId}}}}}}
    ]
  }) {
        subject {
          group {
            name
          }
        }
      }
}
`;

export const CHECK_USER = gql`
  query checkUser($emailId: String!) {
    user(where: { email_id: { _eq: $emailId } }) {
      id
      hpId
      email_id
    }
  }
`;

export const SERVICE_DISCOVERY_DOCUMENT = gql`
query ServiceDiscoveryInfo($id: uuid) {
  servicediscovery(where: {id: {_eq: $id}}) {
    id
    name
    display_name
    developer_group_id
    approver_group_id
    support_group_id
    documentation
    environments {
      tenant_environment_id
      environment {
        id
        name
        attributes
      }
    }
  }
    product(where: {id: {_eq: $id}}) {
    id
    developer_group {
      users {
        user_id
        user {
          id
          user_name
          display_name
          email_id
        }
      }
    }
  }
}
`;

export const ADD_USER = gql`
  mutation CreateUser(
    $hpIdUserId: String
    $userName: String!
    $displayName: String
    $hpIdResponse: jsonb
    $email: String
  ) {
    insert_user(
      objects: [
        {
          hpId: $hpIdUserId
          user_name: $userName
          display_name: $displayName
          email_id: $email
          hpIdResponse: $hpIdResponse
        }
      ]
    ) {
      returning {
        id
      }
    }
  }
`;

export const ADD_USER_TENANT_ASSOCIATION = gql`
  mutation CreateUserTenantAssociation($userId: uuid, $tenantId: uuid) {
    insert_user_tenant(objects: [{ user_id: $userId, tenant_id: $tenantId }]) {
      returning {
        id
      }
    }
  }
`;

export const ADD_USER_TO_GROUP = gql`
  mutation addUserToGroup($insertUser: [user_group_insert_input!]!) {
    insert_user_group(objects: $insertUser) {
      returning {
        user_id
      }
    }
  }
`;

export const GET_USER_AND_GROUP_ID = gql`
  query getUserAndGroupId($emailId: String!, $groupName: String!) {
    user(where: { user_name: { _eq: $emailId } }) {
      id
    }

    group(where: { name: { _eq: $groupName } }) {
      id
    }
  }
`;

export const GET_CREATED_BY_USER_ID = gql`
query getCreateByUserId($emailId: String) {
  user(where: {email_id: {_eq: $emailId}}) {
    id
   }
}
`;

export const ENABLE_OR_DISABLE_USER_MUTATION = gql`
mutation UserUpdate($userId: uuid!, $userUpdateInput: user_set_input) {
  update_user(where: {id: {_eq: $userId}}, _set: $userUpdateInput) {
    affected_rows
    __typename
  }
}
`;

export const REMOVE_USER_FROM_GROUP = gql`
mutation addRemoveUserGroup($emailId: [String!], $groupName: [String!]) {
  delete_user_group(where: {user: {email_id: {_in: $emailId}}, _and: {group: {name: {_in: $groupName}}}}) {
    affected_rows
  }
}
`;

export const DELETE_POLICY_RESOURCE = gql`
mutation deletePolicyResource($policyResourceIds: [uuid!]!) {
  delete_policy_resource(where: { id: { _in: $policyResourceIds } }) {
    affected_rows
  }
}
`;

export const DISABLE_POLICY_RESOURCE = gql`
mutation disablePolicyResource($policyResourceIds: [uuid!]!) {
  update_policy_resource(where: { id: { _in: $policyResourceIds } }, _set: {is_active: false}) {
      affected_rows
    }
}
`;

export const ENABLE_POLICY_RESOURCE = gql`
mutation disablePolicyResource($policyResourceIds: [uuid!]!) {
  update_policy_resource(where: { id: { _in: $policyResourceIds } }, _set: {is_active: true}) {
      affected_rows
    }
}
`;

export const GET_EXTENDED_USERS = gql`
query GetAppAllUsers($appName: [String!], $productName: [String!]) {
  policy_resource(
    where: {
      _and: [
        {
          _or: [
            { resource: { app: { name: { _in: $appName } } } }
            { resource: { product: { name: { _in: $productName } } } }
          ]
        }{
          _or: [
           { policy: { kind: { _eq: "group" } } }
           { policy: { kind: { _eq: "user" } } }
          ]
        }
      ]
    }
  ) {
    policy {
      subject {
        user{
          display_name
          email_id
        }
        group {
          name
          users {
            user {
              email_id
              display_name
            }
          }
        }
      }
    }
    resource{
      app{
        name
        display_name
      }
    }
    taxon_allow {
      value
      taxonomy {
        name
      }
    }
  }
}
`;

export const GET_APP_SUBJECT_NAME_DOCUMENT = gql`
query getAppSubjectName($appProductId: uuid, $subjectId: uuid, $isAppProduct:Boolean! = false) {
  app_product(where: {id: {_eq: $appProductId}}) @include(if: $isAppProduct) {
    id
    app {
      id
      display_name
    }
  }
  app(where: {id: {_eq: $subjectId}}) {
    id
    display_name
  }
  domain(where: {id: {_eq: $subjectId}}) {
    id
    display_name
  }
  group(where: {id: {_eq: $subjectId}}) {
    id
    display_name
  }
  persona(where: {id: {_eq: $subjectId}}) {
    id
    display_name
  }
  user(where: {id: {_eq: $subjectId}}) {
    id
    display_name
    email_id
  }
}

`;

export const UPDATE_REQUESTED_VALUE_MUTATION = gql`
mutation RequestEdit($reqId: uuid!, $requested_value: jsonb) {
  update_request(where: { id: { _eq: $reqId } }, _set: {requested_value: $requested_value}) {
    affected_rows
  }
}
`;

export const UPDATE_REQUEST = gql`
mutation requestUpdate($contitionValue: timestamptz!, $updateValue: request_set_input) {
  update_request( where: { requested_on: { _lte: $contitionValue }, status: { _in: ["new","inprogress","pending"] }, notification_data: { _is_null: true } }, _set: $updateValue) {
    affected_rows
  }
}
`;

export const REQUEST_NOTIFICATION = gql`
mutation requestUpdate($contitionValue: uuid!, $updateValue: request_set_input) {
  update_request( where: {id: { _eq: $contitionValue } }, _set: $updateValue) {
    affected_rows
  }
}
`;

export const GET_REQUEST = gql`
query getRequest{
  request(where: {notification_data: { _is_null: false }, status: { _in: ["new","inprogress","pending"] } }){
    id
    notification_data
  }
}
`;

export const CHECK_GROUPS_EXISTS = gql`
query checkGroupsExists($group: [String!]){
  group(where: {name: {_in: $group }}){
    id
    name
  }
}
`;

export const FETCH_APP_GROUPS = gql`
query fetchAppGroups($appName: [String!]) {
  group(where: {_and: [{is_system: {_eq: false}}, {apps: {app: {name: {_in: $appName}}}}]}) {
    name
  }
}
`;

export const FETCH_ALL_APP_GROUPS = gql`
query fetchAllAppGroups($appName: [String!]) {
  group(where: {apps: {app: {name: {_in: $appName}}}}) {
    name
  }
}
`;

export const FETCH_APP_POLICY_RESOURCE = gql`
query PolicyAppResourceExists($policyId: uuid!, $appName: [String!]) {
  policy_resource(where: {_and: [{policy_id: {_eq: $policyId}}, {resource: {app: {name: {_in: $appName}}}}]}) {
    id
  }
}
`;

export const GET_POLICY_DETAILS = gql`
query PolicyInfoEdit($policyId: uuid) {
  policy(where: { id: { _eq: $policyId } }) {
    id
    kind
    notes
    subject {
      subject
    }
    statements {
      id
      is_active
      app_product_id
      scope {
        product_permission_id
      }
      taxon_allow {
        value
        taxonomy {
          id
          name
        }
      }
      taxon_deny {
        value
        taxonomy {
          id
          name
        }
      }
    }
  }
}
`;

export const GET_APP_DETAILS = gql`
query ApplicationInfoEdit($appId: uuid) {
  app(where: { id: { _eq: $appId } }) {
    id
    name
    display_name
    description
    attributes
    group_behaviour
    environments {
      app_id
      tenant_environment_id
      attributes
    }
    groups {
      is_system
      kind
      group_id
      app_id
      group {
        id
        users {
          user_id
        }
      }
    }
    products {
      environment {
        id
        name
      }
      id
      tenant_environment_id
      product {
        id
        display_name
        name
      }
      app_id
      attributes
    }
    personas {
      persona_id
      app_id
      persona {
        id
        display_name
        name
      }
    }
  }
}
`

export const GET_PRODUCT_DETAILS = gql`
query ProductInfoEdit($productId: uuid) {
  product(where: { id: { _eq: $productId } }) {
    id
    display_name
    name
    description
    attributes
    developer_group_id
    approver_group_id
    support_group_id
    developer_group {
      id
      display_name
      created_on
      is_system
      createdBy {
        id
        display_name
      }
      users {
        user_id
      }
    }
    support_group {
      id
      display_name
      created_on
      is_system
      createdBy {
        id
        display_name
      }
    }
    approver_group {
      id
      display_name
      created_on
      is_system
      createdBy {
        id
        display_name
      }
    }
    permissions {
      id
      name
      description
      rules {
        id
        method
        urlPattern
        constraints
      }
    }
    taxonomies(order_by: { attributes: asc }) {
      id
      name
      display_name
      description
      attributes
    }
    status
    environments {
      product_id
      tenant_environment_id
      attributes
    }
    documentation
  }
}
`

export const GET_POLICY_RESOURCE = gql`
query GetPolicyByResourceId($resourceId: uuid!){
  policy_resource(where: {id: {_eq: $resourceId }}){
    policy_id
  }
}
`;

export const GET_USER_EMAIL = gql`
query GetUserEmail($updatedBy: uuid!){
  user(where: {id: {_eq: $updatedBy}}) {
    email_id,
    id,
    display_name
  }
}
`;

export const CREATE_AUDIT_LOG = gql`
  mutation AuditLogNew(
    $entityType: String!
    $entityId: uuid!
    $newData: jsonb
    $updatedBy: String!
    $updatedOn: timestamptz!
    $traceId: String!,
    $entityName: String
  ) {
    insert_audit_log(
      objects: [
        {
          entity_type: $entityType
          entity_id: $entityId
          new_data: $newData
          updated_by: $updatedBy
          updated_on: $updatedOn
          trace_id: $traceId
          entity_name: $entityName
        }
      ]
    ) {
      returning {
        id
      }
    }
  }
`;


export const GET_PRODUCT_ID_BY_PERMISSION_ID = gql`
query getProductIdbyPermissionId($id: uuid!) {
  product_permission(where: {id: {_eq: $id}}) {
    product_id
  }
}
`;

export const GET_ENTITY_NAME = gql`
query getEntityName($entityId: uuid) {
  policy(where: {id: {_eq: $entityId}}) {
    subject {
      app {
        name
      }
      domain {
        name
      }
      group {
        name
      }
      persona {
        name
      }
      user {
        email_id
      }
    }
  }
  app(where: {id: {_eq: $entityId}}) {
    name
  }
  product(where: {id: {_eq: $entityId}}) {
    name
  }
  user(where: {id: {_eq: $entityId}}) {
    user_name
  }
}
`;

export const GET_ENTITY_NAME_BY_POLICY_SUBJECT = gql`
query getAppSubjectName($subjectId: uuid) {
  app(where: {id: {_eq: $subjectId}}) {
    name
  }
  domain(where: {id: {_eq: $subjectId}}) {
    name
  }
  group(where: {id: {_eq: $subjectId}}) {
    name
  }
  persona(where: {id: {_eq: $subjectId}}) {
    name
  }
  user(where: {id: {_eq: $subjectId}}) {
    email_id
  }
}
`;

export const GET_APP_ATTRIBUTES_BY_APP_NAME = gql`
query getAppAttributesByAppName($appName: String) {
  app(where: {name: {_eq: $appName}}) {
    attributes
  }
}
`;

export const GET_GROUP_DETAILS = gql`
query getGroupDetails($entityId:uuid) {
  group(where: {id: {_eq: $entityId}}){
    display_name
    name
    __typename  
  }
}
`;

export const GET_PERSONA_DETAILS = gql`
query getTenantDetails($entityId:uuid) {
  persona(where: {id: {_eq: $entityId}}){
    name
    display_name
    __typename  
    
  }
}
`;
export const GET_USER_DETAILS = gql`
query getUserDetails($entityId:uuid) {
  user(where: {id: {_eq: $entityId}}){
    user_name
    display_name
    is_active
    is_enabled 
  }
}
`;

export const GET_PRODUCT_ALL_TAXON_ALLOWED = gql`
query getProductAllTaxonsallowed($productName: String, $taxonamyName: String) {
  product(where: {name: {_eq: $productName}}) {
    id,
    taxonomies(where: {name: {_eq: $taxonamyName}}) {
      name
      taxonsAllowed {
        policy_resource_id,
        product_taxonomy_id,
        value
      }
    }
  }
}
`;
export const GET_All_USERS = gql`
  query GET_ALL_USER_LIST {
    user {
      email_id
    }
    user_aggregate {
      aggregate {
        count
      }
    }
  }
`;
export const GET_APP_LIST_FOR_USERS = gql`
  query GET_APP_LIST_FOR_USERS {
    app(
      where: {
        attributes: {
          _contains: [{ name: "disabled-inactive-user", value: "true" }]
        }
      }
    ) {
      tenant_id
      name
      display_name
      attributes
    }
  }
`;

export const UPDATE_REQUEST_COMMENTS = gql`
mutation RequestEdit($requestId: uuid!, $requestUpdateInput: jsonb!) {
  update_request(where: { id: { _eq: $requestId } }, _set: {comments: $requestUpdateInput}) {
    affected_rows
  }
}
`;
