import { Client } from "@elastic/elasticsearch";
import { Logger, LibLogger } from "../lib/lambda-logger/index";
import { Configuration } from "../config";

export class ESClient {
  esClient: Client;
  log: Logger;
  esIndex: string;
  baseESIndex: string;

  constructor(log: Logger = LibLogger.Instance) {
    const elasticUrl = Configuration().elasticSearchUrl;
    this.log = log;

    this.esClient = new Client({
      node: elasticUrl,
    });
  }

  async searchByQuery(emailIds: any): Promise<any[]> {
    const batchSize = 10000;
    const totalEmailIds = emailIds.length;
    let processedEmailIds = 0;
    const allUsersWithAppCount: any[] = [];

    while (processedEmailIds < totalEmailIds) {
      const batchEmailIds = emailIds.slice(
        processedEmailIds,
        processedEmailIds + batchSize
      );
      const params = {
        index: "paladin-policy-service-dev-*",
        body: {
          _source: ["time", "data.emailId", "data.appName", "data.tenantId"],
          size: 0,
          query: {
            bool: {
              must: [
                {
                  terms: {
                    "actor.id.keyword": batchEmailIds,
                  },
                },
              ],
            },
          },
          aggs: {
            uniqueEmailIds: {
              terms: {
                field: "data.emailId.keyword",
                size: batchSize,
              },
              aggs: {
                uniqueApps: {
                  terms: {
                    field: "data.appName.keyword",
                    size: batchSize,
                  },
                  aggs: {
                    lastLogin: {
                      top_hits: {
                        _source: {
                          includes: [
                            "data.appName",
                            "time",
                            "data.emailId",
                            "data.tenantId",
                          ],
                        },
                        size: 1,
                        sort: [
                          {
                            time: {
                              order: "desc",
                            },
                          },
                        ],
                      },
                    },
                  },
                },
              },
            },
          },
        },
      };

      this.log.info("Searching with params", params);
      try {
        const result: any = await this.esClient.search(params);
        this.log.info("search params result", result);

        const uniqueEmailIds: any[] =
          result.body.aggregations.uniqueEmailIds.buckets;
        const usersWithAppCount: any[] = uniqueEmailIds.map(
          (emailBucket: any) => {
            const emailId = emailBucket.key;
            const appBuckets: any[] = emailBucket.uniqueApps.buckets;
            const appInfo: any[] = appBuckets.map((appBucket: any) => {
              const appName = appBucket.key;
              const lastLoginInfo = appBucket.lastLogin.hits.hits[0]._source;
              const tenantId = lastLoginInfo.data.tenant_id;
              return {
                appName,
                tenantId,
                lastLoginInfo,
              };
            });
            return {
              emailId,
              apps: appInfo,
            };
          }
        );

        allUsersWithAppCount.push(...usersWithAppCount);
        processedEmailIds += batchSize;
      } catch (err) {
        this.log.error("Error while querying Elasticsearch: " + err);
        return [];
      }
    }

    return allUsersWithAppCount;
  }
}
