import { Logger, LambdaLogger } from "../lib/lambda-logger";
const axios = require("axios");
import qs from "qs";
var fs = require("fs");
var unzipper = require("unzipper");
var replace = require("replace-in-file");
var zipLib = require("zip-lib");
const request = require("request-promise");
const FormData = require("form-data");
import { S3 } from "aws-sdk";

export class ApigeeApiService {
  private accessToken: string = "";
  private apigeeAccessTokenUrl: string =
    "https://hp.login.apigee.com/oauth/token";
  private apigeeAccessTokenUrlResetKey: string =
    "https://hp.login.apigee.com/oauth/token";
  private apigeeBaseUrl: string =
    "https://api.enterprise.apigee.com/v1/organizations/hpcorp-ob/";
  private apigeeSecretResetUrl: string =
    "https://api.enterprise.apigee.com/v1/organizations/hpcorp-ob/developers/MPS-RCB-AUTH@HP.onmicrosoft.com/apps/";

  public log: Logger = LambdaLogger.getInstance("ApigeeApiService");

  constructor() {
    if (process.env.stage === "prod") {
      this.apigeeAccessTokenUrlResetKey =
        "https://core.api.hp.com/clientServices/v1/token";
      this.apigeeBaseUrl =
        "https://api.enterprise.apigee.com/v1/organizations/hpcorp/";
      this.apigeeSecretResetUrl =
        "https://core.api.hp.com/clientServices/v1/clients/clientIdPlaceHolder/secret";
    }
  }

  public async generateAccessToken(
    username,
    password,
    isResetKeyOperation: boolean
  ): Promise<any> {
    this.log.info("Getting token from apigee server");

    var url = this.apigeeAccessTokenUrl;
    if (isResetKeyOperation) {
      url = this.apigeeAccessTokenUrlResetKey;
    }

    let headers: any = {};
    let formData: any = {};

    if (process.env.stage !== "prod") {
      headers = {
        Authorization: "Basic ZWRnZWNsaTplZGdlY2xpc2VjcmV0",
        Accept: "application/json;charset=utf-8",
        "content-type": "application/x-www-form-urlencoded",
      };

      formData = {
        username: username,
        password: password,
        grant_type: "password",
        response_type: "access_token",
      };
    } else {
      if (isResetKeyOperation) {
        const basicAuthToken = Buffer.from(
          `${username}:${password}`,
          "utf8"
        ).toString("base64");
        headers = {
          "content-type": "application/x-www-form-urlencoded",
          Authorization: `Basic ${basicAuthToken}`,
        };
  
        formData = {
          grant_type: "client_credentials",
        };
      } else {
        headers = {
          Authorization: "Basic ZWRnZWNsaTplZGdlY2xpc2VjcmV0",
          Accept: "application/json;charset=utf-8",
          "content-type": "application/x-www-form-urlencoded",
        };
  
        formData = {
          username: username,
          password: password,
          grant_type: "password",
          response_type: "access_token",
        };
      }
    }

    const options = {
      method: "POST",
      headers: headers,
      data: qs.stringify(formData),
      url: url,
    };

    let response: any = {};

    try {
      response = await axios(options);
      if (response && response.data) {
        this.log.info("Got the token from apigee");
        this.accessToken = response.data.access_token;
      }
    } catch (error) {
      this.log.error("Error while getting apigee token: " + error);
      response.result = false;
    }

    return response;
  }

  public async deleteCurrentClienId(
    appName: string,
    clientId: string
  ): Promise<any> {
    let url: string = this.apigeeSecretResetUrl + appName + "/keys/" + clientId;
    let response = await this.makeApigeeApiRequest(url, "delete", "");
    return response;
  }

  public async createNewClientId(
    appName: string,
    clientId: string,
    newSecret: string
  ): Promise<any> {
    let url: string = this.apigeeSecretResetUrl + appName + "/keys/create";
    let body: any = {
      consumerKey: clientId,
      consumerSecret: newSecret,
    };

    let response = await this.makeApigeeApiRequest(url, "POST", body);
    return response;
  }

  public async addApiProductsToClientId(
    appName: string,
    clientId: string,
    products: string[]
  ): Promise<any> {
    let url: string = this.apigeeSecretResetUrl + appName + "/keys/" + clientId;
    let body: any = {
      apiProducts: products,
    };

    let response = await this.makeApigeeApiRequest(url, "POST", body);
    return response;
  }

  public async updateAppAttribute(
    appName: string,
    attributeName: string,
    attributeValue: string
  ) {
    let url: string =
      this.apigeeSecretResetUrl + appName + "/attributes/" + attributeName;
    let body: any = {
      value: attributeValue,
    };

    let response = await this.makeApigeeApiRequest(url, "POST", body);
    return response;
  }

  public async resetClientSecret(
    clienId: string,
    encryptedSecret: string
  ): Promise<any> {
    let headers: any = {
      Authorization: "Bearer " + this.accessToken,
      encryptedSecret: encryptedSecret,
    };

    this.apigeeSecretResetUrl = this.apigeeSecretResetUrl.replace(
      "clientIdPlaceHolder",
      clienId
    );
    let response = await this.makeApigeeApiRequest(
      this.apigeeSecretResetUrl,
      "POST",
      "",
      headers
    );
    return response;
  }

  public async createTargetServer(
    serverName: string,
    targetHostname: string,
    env: any
  ) {
    let url: string =
      this.apigeeBaseUrl + "environments/" + env + "/targetservers";
    var body: any = {
      host: targetHostname,
      isEnabled: true,
      name: serverName,
      port: 443,
      sSLInfo: {
        ciphers: [],
        clientAuthEnabled: "false",
        enabled: "true",
        ignoreValidationErrors: false,
        protocols: [],
        trustStore: "",
      },
    };

    var response = await this.makeApigeeApiRequest(url, "POST", body);
    return response;
  }

  public async updateTargetServer(
    serverName: string,
    targetHostname: string,
    env: any
  ) {
    let url: string =
      this.apigeeBaseUrl +
      "environments/" +
      env +
      "/targetservers/" +
      serverName;
    var body: any = {
      host: targetHostname,
      isEnabled: true,
      port: 443,
      sSLInfo: {
        ciphers: [],
        clientAuthEnabled: "false",
        enabled: "true",
        ignoreValidationErrors: false,
        protocols: [],
        trustStore: "",
      },
    };

    var response = await this.makeApigeeApiRequest(url, "POST", body);
    return response;
  }

  public async getProxyDetails(name: string) {
    let url: string = this.apigeeBaseUrl + "apis/" + name;
    var response = await this.makeApigeeApiRequest(url, "GET", "");
    return response;
  }

  public async deployProxy(name: string, env: any, revision: string) {
    let url: string =
      this.apigeeBaseUrl +
      "environments/" +
      env +
      "/apis/" +
      name +
      "/revisions/" +
      revision +
      "/deployments?override=true";
    var response = await this.makeApigeeApiRequest(url, "POST", "");
    return response;
  }

  public async getProductDetails(name: string) {
    let url: string = this.apigeeBaseUrl + "apiproducts/" + name;
    var response = await this.makeApigeeApiRequest(url, "GET", "");
    return response;
  }

  public async getStatistics(
    select: string,
    timeRange: string,
    timeUnit: string,
    proxyName: string,
    env: string,
  ) {
    let queryParam: string =
      "select=" + select + "&timeRange=" + timeRange + "&timeUnit=" + timeUnit;
    // let queryParam: string =
    //   "select=sum(message_count)&timeRange=2/8/2021%2000:00~2/10/2021%2023:59&timeUnit=day";
    let url: string =
      this.apigeeBaseUrl +
      "environments/" +
      env +
      "/stats/apiproxy/" +
      proxyName +
      "?" +
      queryParam;

    var response = this.makeApigeeApiRequest(url, "GET", "");
    return response;
  }

  public async deployProduct(name: string, payload: any, env: any) {
    let url: string = this.apigeeBaseUrl + "apiproducts/" + name;
    var body: any = {};
    body.approvalType = "auto";
    body.attributes = payload.attributes;
    body.name = name;
    body.displayName = payload.displayName;
    body.description = payload.description;
    body.environments = payload.environments;
    body.environments.push(env);
    body.proxies = payload.proxies;

    var response = this.makeApigeeApiRequest(url, "POST", body);
    return response;
  }

  public updateProduct(name: string, payload: any) {
    let url: string = this.apigeeBaseUrl + "apiproducts/" + name;
    var body: any = {};
    body.approvalType = "auto";
    body.attributes = payload.attributes;
    body.name = name;
    body.displayName = payload.displayName;
    body.description = payload.description;
    body.environments = payload.environments;
    body.proxies = payload.proxies;

    var response = this.makeApigeeApiRequest(url, "POST", body);
    return response;
  }

  public async createProduct(
    displayName,
    shortName,
    description,
    env,
    attributes
  ) {
    let url: string = this.apigeeBaseUrl + "apiproducts";
    var body: any = {
      approvalType: "auto",
      environments: [],
      proxies: [],
    };
    body.displayName = displayName;
    body.name = shortName;
    body.description = description;
    body.attributes = attributes;
    body.attributes.push({
      name: "short-name",
      value: shortName,
    });
    body.environments.push(env);
    body.proxies.push(shortName + "-proxy");

    var response = this.makeApigeeApiRequest(url, "POST", body);
    return response;
  }

  public async createApiProxyPackage(productName) {
    let s3 = new S3();
    const params = {
      Bucket: "paladin-apigee-proxy-template",
      Key: "proxy_template.zip",
    };

    var data = await s3.getObject(params).promise();

    fs.writeFileSync("/tmp/proxy_template.zip", data.Body);

    let folderPath = "/tmp/";
    var proxyZipFileName = folderPath + productName + "-proxy.zip";
    if (process.env.stage === "prod") {
      var proxyZipFileName =
        folderPath + "hpcorp-ob-" + productName + "-proxy.zip";
    }

    return new Promise(function (resolve) {
      fs.createReadStream(folderPath + "proxy_template.zip").pipe(
        unzipper.Extract({ path: folderPath }).on("close", function () {
          var apiProxyFolderpath = folderPath + "proxy_template/apiproxy/";
          fs.renameSync(
            apiProxyFolderpath + "proxy_template.xml",
            apiProxyFolderpath + productName + "-proxy.xml"
          );

          var replaceOptions = {
            files: apiProxyFolderpath + productName + "-proxy.xml",
            from: ["proxy_base_path", "proxy_display_name"],
            to: [productName, productName],
          };
          replace.sync(replaceOptions);

          replaceOptions = {
            files: apiProxyFolderpath + "targets/default.xml",
            from: ["server_name"],
            to: [productName + "-server"],
          };
          replace.sync(replaceOptions);

          replaceOptions = {
            files: apiProxyFolderpath + "proxies/default.xml",
            from: ["proxy_base_path"],
            to: [productName],
          };
          replace.sync(replaceOptions);

          zipLib
            .archiveFolder(folderPath + "proxy_template", proxyZipFileName)
            .then(function () {
              resolve({
                result: "success",
              });
            });
        })
      );
    });
  }

  public async importApiProxy(shortName) {
    let proxyName: string = shortName + "-proxy";
    let folderPath = "/tmp/";
    var proxyZipFile = folderPath + shortName + "-proxy.zip";

    var url = this.apigeeBaseUrl + "/apis?action=import&name=" + proxyName;

    var formData = new FormData();
    formData.append("file", fs.createReadStream(proxyZipFile));

    let response = await axios.post(url, formData, {
      headers: {
        Authorization: "Bearer " + this.accessToken,
        "Content-Type": "multipart/form-data",
      },
    });

    fs.unlinkSync(folderPath + shortName + "-proxy.zip");

    return response.data;
  }

  public async getAppDetails(name: string) {
    let url: string = this.apigeeSecretResetUrl + name;
    var response = await this.makeApigeeApiRequest(url, "GET", "");
    return response;
  }

  public async createApp(displayName, shortName) {
    var body: any = {
      attributes: [],
    };

    body.name = shortName;
    body.attributes.push({
      name: "DisplayName",
      value: displayName,
    });

    var currentDateInSeconds = Math.round(Date.now() / 1000);
    body.attributes.push({
      name: "secretCreatedDate",
      value: currentDateInSeconds,
    });

    var oneDayTimeSpanInSeconds = 24 * 60 * 60;
    var secretExpiryDate = currentDateInSeconds + 90 * oneDayTimeSpanInSeconds;
    body.attributes.push({
      name: "secretExpiryDate",
      value: secretExpiryDate,
    });

    body.attributes.push({
      name: "oldSecret",
      value: "",
    });

    var response = this.makeApigeeApiRequest(
      this.apigeeSecretResetUrl,
      "POST",
      body
    );
    return response;
  }

  public async AddKvmEntry(
    name: string,
    appType,
    secretRotationOptIn,
    secretPath,
    clientId: string,
    env: any,
    tenantId: string,
    tokenExpiryTime: string
  ) {
    var url: string =
      this.apigeeBaseUrl +
      "environments/" +
      env +
      "/keyvaluemaps/hpcorp-ob-paladin-kvm/entries";

    var payload: any = {
      webAppTypeValue:
        '{"hpid-auth":"","hponeid-auth":"","p-tenant-id":"tenantIdPlaceHolder","consumerKey":"consumerkeyPlaceHolder"}',
      serviceTypeValue:
        '{"p-tenant-id":"tenantIdPlaceHolder","consumerKey":"consumerkeyPlaceHolder","secretRotationOptIn":secretRotationOptInPlaceHolder,"secretPath":"secretPathPlaceHolder","tokenExpiryTime":"tokenExpiryTimePlaceHolder"}',
    };
    
    var data: string;
    if (appType === "web") {
      data = payload.webAppTypeValue;
      data = data.replace("consumerkeyPlaceHolder", clientId);
      data = data.replace("tenantIdPlaceHolder", tenantId);
    } else {
      data = payload.serviceTypeValue;
      data = data.replace("consumerkeyPlaceHolder", clientId);
      data = data.replace(
        "secretRotationOptInPlaceHolder",
        secretRotationOptIn
      );
      data = data.replace("secretPathPlaceHolder", secretPath);
      data = data.replace("tenantIdPlaceHolder", tenantId);
      data = data.replace("tokenExpiryTimePlaceHolder", tokenExpiryTime);
    }

    var body: any = {
      name: name,
      value: data,
    };

    var response = this.makeApigeeApiRequest(url, "POST", body);
    return response;
  }

  public async DeleteKvmEntry(name: string, env: any) {
    var url: string =
      this.apigeeBaseUrl +
      "environments/" +
      env +
      "/keyvaluemaps/hpcorp-ob-paladin-kvm/entries/" +
      name;
    var response = this.makeApigeeApiRequest(url, "DELETE", "");
    return response;
  }

  public async getKvmEntry(name: string, env: any) {
    var url: string =
      this.apigeeBaseUrl +
      "environments/" +
      env +
      "/keyvaluemaps/hpcorp-ob-paladin-kvm/entries/" +
      name;
    var response = this.makeApigeeApiRequest(url, "GET", "");
    return response;
  }

  private async makeApigeeApiRequest(
    url: string,
    method: string,
    body: string,
    incomingHeader?: any
  ): Promise<any> {
    let headers: any = {
      Authorization: "Bearer " + this.accessToken,
    };

    if (incomingHeader) {
      headers = incomingHeader;
    }

    const options = {
      method: method,
      headers: headers,
      url: url,
      data: "",
    };

    if (method.toLowerCase() === "post") {
      options.data = body;
    }

    let response: any = {};

    try {
      if (method.toLowerCase() == "delete" || method.toLowerCase() == "get") {
        response = await request(options);
      } else {
        response = await axios(options);
      }

      if (response) {
        this.log.info("Successfully got the response for " + url);
      }
    } catch (error) {
      this.log.error(
        "Error while making call to: " + url + ". Error is: " + error
      );
      response.result = false;
    }

    return response;
  }
}
