import { SNS } from "aws-sdk";
const axios = require("axios");
import { Logger, LambdaLogger } from "../lib/lambda-logger/index";
import { clientApiKey } from "../enviroments/local_environment";
import { ApolloQueryResult } from "apollo-boost";
import { Configuration } from "../config";
import { GET_APP_DETAILS_BY_NAME, GET_TENANT_DETAILS_BY_ID } from "../graphqlQueries/request-info.query";

interface PublishInput {
    Subject: string,
    Message: string,
    TopicArn: string
}

export class PublisherService {

    log: Logger = LambdaLogger.getInstance("PublisherService");

    public async publishResetKeyStatus(appName: string, autoRotationEnabled: boolean, clientId: string, status: string) {

        const tenantId = await this.getTenantIdByAppName(appName);
        if(tenantId) {
          const tenantTemplateName = await this.getTenantTemplateNameByTenantId(tenantId);
          if(tenantTemplateName) {
            const params: PublishInput = {
                Subject: tenantTemplateName + ' Auth - ' + process.env.stage?.toUpperCase() + ' : Reset secret key status for ' + appName,
                Message: 'Details: \n\n - App Name: ' + appName + '\n - Client ID: ' + clientId + '\n - Auto rotation enabled: ' + autoRotationEnabled + '\n - Status: ' + status,
                TopicArn: Configuration().configServiceSNS
            };
    
            await this.publishEventToMSTeams(params.Subject, params.Message);
            await this.publishEventToSNS(params);
          }
        }
        return;
    }

    private async publishEventToSNS(params: PublishInput) {

        const options: SNS.ClientConfiguration = {
            endpoint: "https://sns.us-west-2.amazonaws.com",
            region: "us-west-2",
        };

        const snsClient = new SNS(options);

        try {
            const result = await snsClient.publish(params).promise();
            if (result) {
                this.log.info("Successfully published event." + JSON.stringify(result));
                return;
            } else {
                this.log.error("Failed to publish event" + JSON.stringify(result));
                return;
            }
        } catch (error) {
            this.log.error("Error while publishing event: " + error);
            return;
        }
    }

    private async publishEventToMSTeams(title: string, text: string) {

        const data = {
            "@context": "https://schema.org/extensions",
            "@type": "MessageCard",
            themeColor: "64a837",
            title,
            text
        }
        const config = {
            headers: {
                "Content-Type": "application/json",
            },
        }

        try {
            const result = await axios.post('' + Configuration().configServiceAlertsWebHook, data, config);
            if (result) {
                this.log.info("Successfully published event to MSTeams");
                return;
            } else {
                this.log.error("Failed to publish event to MSTeams");
                return;
            }
        } catch (error) {
            this.log.error("Error while publishing event to MSTeams: " + error);
            return;
        }
    }

    public async getTenantTemplateNameByTenantId(tenantId: any) {
        const tenantDetails = await this.getDataFromQuery(GET_TENANT_DETAILS_BY_ID, {
          tenantId
        });
    
        if (tenantDetails.tenant && tenantDetails.tenant[0].attributes.length > 0) {
            for await (const attribute of tenantDetails.tenant[0].attributes) {
                if (attribute.name == 'tenant-template-name') {
                return attribute.value
                }
              }
        }
          return "";
      }
    
      public async getTenantIdByAppName(appName: String) {
        const getappTenantId = await this.getDataFromQuery(GET_APP_DETAILS_BY_NAME, {
          appName
        });
        if (getappTenantId.app && getappTenantId.app.length > 0) {
          return getappTenantId.app[0].tenant_id;
        }
        return;
      }

      private async getDataFromQuery(
        query: any,
        variables: any
      ): Promise<any | undefined> {
        try {
          const result: ApolloQueryResult<any> = await clientApiKey.query({
            query,
            variables,
          });
          this.log.info(
            "Received DB Call Result: ",
            result && result.data ? JSON.stringify(result.data) : "NULL"
          );
          return result.data;
        } catch (err) {
          this.log.error("Error while querying data from DB", err);
          return;
        }
      }
}
